<div class="unfold-preview">
    <div class="ckanext-jstree-search">
        <div class="jstree-search--input">
            <label class="d-none" for="field-sitewide-search">{{ _('Search') }}</label>
            <input id="jstree-search" class="form-control" type="text" name="jstree-search"
                placeholder="{{ _('Search') }}" aria-label="{{ _('Search') }}" />
            <button class="btn search-icon" type="submit" aria-label="{{ _('Search icon') }}"><i
                    class="fa fa-search"></i></button>
        </div>

        <div class="jstree-search--clear">
            <button class="btn btn-primary" id="jstree-search-clear">
                <i class="fa fa-close"></i>
                {{ _("Clear") }}
            </button>
        </div>
    </div>

    <div id="archive-tree" data-module="unfold-init-jstree" data-module-resource-id="{{ resource.id }}" data-module-resource-view-id="{{ resource_view.id }}" >
        <div class="archive-tree--spinner">
            <img src="{{ h.url_for_static('/spinner.gif') }}" width="20">
            {{ _("The archive tree is currently being initialized. Please wait...") }}
        </div>
    </div>

    <div id="archive-tree-error" style="display: none">
        <img src="{{ h.url_for_static('/error.gif') }}" width="30">
        <span></span>
    </div>
</div>

{% asset 'unfold/unfold-js' %}
{% asset 'unfold/unfold-css' %}
