.ckanext-jstree-search{margin-bottom:12px;display:flex;gap:12px}.ckanext-jstree-search .jstree-search--input{display:flex;position:relative;flex-basis:350px;height:35px}.ckanext-jstree-search .jstree-search--input .search-icon{height:35px;display:flex;position:absolute;right:0}.jstree-default>.jstree-no-dots .jstree-closed .jstree-ocl,.jstree-default>.jstree-no-dots .jstree-open .jstree-ocl{background-image:unset;font-style:inherit}.jstree-default>.jstree-no-dots .jstree-closed .jstree-ocl:before,.jstree-default>.jstree-no-dots .jstree-open .jstree-ocl:before{font-family:var(--fa-style-family, "Font Awesome 6 Free");font-weight:var(--fa-style,900)}.jstree-default>.jstree-no-dots .jstree-open .jstree-ocl{background-image:unset}.jstree-default>.jstree-no-dots .jstree-open .jstree-ocl:before{content:""}.jstree-default>.jstree-no-dots .jstree-closed .jstree-ocl:before{content:""}.jstree-default>.jstree-no-dots .jstree-leaf>.jstree-ocl:before,.jstree-table-header:last-of-type .jstree-table-separator{display:none}.jstree-default .jstree-search{background:bisque}.jstree-default .jstree-anchor[aria-expanded=true] .jstree-icon:before{content:""}.jstree-table-wrapper .jstree-table-headerwrapper,.jstree-table-wrapper .jstree-table-midwrapper{width:-webkit-fill-available;width:-moz-available;display:table}#archive-tree-error{padding:10px;border:1px solid #ced4da;border-radius:.25rem}.js-tree-col{min-width:100px!important}
